@@ -1,5 +1,6 @@
 
 
+################################################################################
 ################################################################################
 #
 # IO helpers
@@ -10,9 +11,9 @@
 
 #' File pattern
 #'
-#' Create regexp matching certain file extensions. This is not normally
-#' directly called by an \pkg{opm} user but by the other IO functions of the
-#' package.
+#' Create a regular expression matching certain file extensions. This is not
+#' normally directly called by an \pkg{opm} user but by the other IO functions
+#' of the package.
 #'
 #' @param type Character scalar indicating the file types to be matched by 
 #'   extension.
@@ -21,7 +22,7 @@
 #' @return Character scalar, holding a regular expression.
 #' @family IO-functions
 #' @keywords utilities
-#' @seealso file_ext
+#' @seealso tools::file_ext
 #' @examples
 #' (x <- file_pattern())
 #' (y <- file_pattern(type = "csv", compressed = FALSE))
@@ -43,6 +44,9 @@ file_pattern <- function(type = c("both", "csv", "yaml", "any", "empty"),
 }
 
 
+################################################################################
+
+
 ## NOTE: not an S4 method because dispatch is done manually
 
 #' File pattern
@@ -57,7 +61,7 @@ file_pattern <- function(type = c("both", "csv", "yaml", "any", "empty"),
 #' @return Character scalar, holding a regular expression, or any object, 
 #'   unchanged.
 #' @keywords internal
-#' @seealso file_ext
+#' @seealso tools::file_ext
 #'
 extended_file_pattern <- function(arg, wildcard) {
   if (is.null(arg))
@@ -71,6 +75,7 @@ extended_file_pattern <- function(arg, wildcard) {
 }
 
 
+################################################################################
 ################################################################################
 #
 # Input of single OPM files
@@ -106,12 +111,13 @@ read_old_opm <- function(filename) {
   pos <- which(data == HOUR)
   if (length(pos) != 1L)
     stop("uninterpretable header (maybe because there is not 1 plate per file)")
-  pos <- seq(pos - 1L)
+  pos <- seq_len(pos - 1L)
   
   # Process comments
   comments <- fold_comments(c(FILE, filename, data[pos]))
   names(comments)[names(comments) == "Set up Time"] <- SETUP
-  comments[PLATE_TYPE == "PM 1-"] <- "PM01"
+  comments[PLATE_TYPE] <- map_values(comments[PLATE_TYPE],
+    c(OTH = GEN_III, `PM 1-` = "PM01"))
 
   # Process data
   data <- data[-pos]
@@ -126,6 +132,9 @@ read_old_opm <- function(filename) {
 }
 
 
+################################################################################
+
+
 ## NOTE: not an S4 method because scan() gives an adequate error message
 
 #' Read OmniLog CSV file
@@ -142,20 +151,24 @@ read_new_opm <- function(filename) {
     comment.char = "#", strip.white = TRUE)
   pos <- which(data == HOUR)
   if (length(pos) != 1L)
-    stop("uninterpretable header (maybe because there is not 1 plate per file)")
+    stop("uninterpretable header (maybe there is not 1 plate per file)")
   ncol <- pos + 96L
   if (length(data) %% ncol != 0L)
     stop("wrong number of fields")
-  data <- matrix(data[-seq(ncol)], ncol = ncol, byrow = TRUE,
-    dimnames = list(NULL, data[seq(ncol)]))
-  pos <- seq(pos - 1L)
+  data <- matrix(data[-seq_len(ncol)], ncol = ncol, byrow = TRUE,
+    dimnames = list(NULL, data[seq_len(ncol)]))
+  pos <- seq_len(pos - 1L)
   comments <- structure(.Data = c(filename, data[1L, pos]),
     names = c(FILE, colnames(data)[pos]))
+  comments[PLATE_TYPE] <- map_values(comments[PLATE_TYPE], c(OTH = GEN_III))
   data <- apply(data[, -pos], MARGIN = 2L, FUN = as.numeric)
   new(OPM, measurements = data, metadata = list(), csv_data = comments)
 }
 
 
+################################################################################
+
+
 ## NOTE: not an S4 method because yaml.load_file() gives an adequate error 
 ##   message
 
@@ -185,6 +198,9 @@ read_opm_yaml <- function(filename) {
 }
 
 
+################################################################################
+
+
 ## NOTE: not an S4 method because conversion is done
 
 #' Read single PM file
@@ -202,7 +218,7 @@ read_opm_yaml <- function(filename) {
 #' @family IO-functions
 #' @note \itemize{
 #'   \item The expected CSV format is what is output by the OmniLog(R) device, 
-#'     one plate per file. Other formats, or OmniLog(R) files resaved with 
+#'     one plate per file. Other formats, or OmniLog(R) files re-saved with 
 #'     distinct CSV settings, are not understood. For this reason, if any 
 #'     editing of the files was necessary at all, it is advisable to do this in
 #'     an editor for plain text, not in a spreadsheet program. 
@@ -211,10 +227,14 @@ read_opm_yaml <- function(filename) {
 #'     see the example under \code{\link{split_files}}.
 #'   \item In contrast, input YAML files can contain data from more than one 
 #'     plate.
+#'   \item Plates run in ID mode are automatically detected as such (their 
+#'     plate type is changed from \sQuote{OTH} to the internally used spelling
+#'     of \sQuote{Generation III}). A generation-III plate type can also be 
+#'     forced later on by using \code{\link{gen_iii}}.
 #' }
 #' @references \url{http://www.yaml.org/}
 #' @references \url{http://www.biolog.com/}
-#' @seealso read.csv
+#' @seealso utils::read.csv
 #' @keywords IO
 #' @examples
 #' test.files <- opm_files("testdata")
@@ -253,6 +273,7 @@ read_single_opm <- function(filename) {
 }
 
 
+################################################################################
 ################################################################################
 #
 # Input of multiple OPM files
@@ -308,7 +329,7 @@ read_single_opm <- function(filename) {
 #'   normal running mode.
 #'
 #' @export
-#' @seealso list.files Sys.glob
+#' @seealso base::list.files base::Sys.glob
 #' @family IO-functions
 #' @keywords IO character
 #' @examples
@@ -368,6 +389,9 @@ explode_dir <- function(names,
 }
 
 
+################################################################################
+
+
 #' Opm files
 #'
 #' Get list of files from the \pkg{opm} package of interest for the user.
@@ -377,7 +401,7 @@ explode_dir <- function(names,
 #'   \describe{
 #'     \item{scripts}{R script files for non-interactive uses of the 
 #'        \pkg{opm} package, particularly for the batch processing of many
-#'        files. When called withput input arguments or with the
+#'        files. When called without input arguments or with the
 #'        \sQuote{-h} switch, the scripts output usage information.}
 #'     \item{testdata}{Files as output by the OmniLog(R) device for testing
 #'        data input and metadata management.}
@@ -386,7 +410,7 @@ explode_dir <- function(names,
 #' @return Character vector of filenames.
 #' @note This might fail with unusual installations of the \pkg{opm} package.
 #' @family IO-functions
-#' @seealso list.files
+#' @seealso base::list.files
 #' @keywords utilities
 #' @examples
 #' (x <- opm_files("scripts"))
@@ -407,6 +431,9 @@ opm_files <- function(what = c("scripts", "testdata")) {
 }
 
 
+################################################################################
+
+
 ## Not an S4 method because conversion is done
 
 #' Read multiple PM files at once
@@ -417,9 +444,10 @@ opm_files <- function(what = c("scripts", "testdata")) {
 #' excluded using \code{include} and/or \code{exclude}).
 #'
 #' @param names Character vector with names of files in one of three formats
-#'   accepted by \code{read_opm} ((see above), or names of directories 
+#'   accepted by \code{\link{read_opm}}, or names of directories 
 #'   containing such files, or both; or convertible to such a vector. See the
-#'   \code{include} argument below and \code{\link{explode_dir}} for how to 
+#'   \code{include} argument of \code{\link{read_opm}} and 
+#'   \code{\link{explode_dir}} for how to 
 #'   select subsets from the input files or directories.
 #'
 #' @param convert Character scalar. If \sQuote{no}, always return a list. If
@@ -428,7 +456,9 @@ opm_files <- function(what = c("scripts", "testdata")) {
 #'   behaves like \sQuote{yes} but does not result in an error message if
 #'   conversion to OPMS is impossible; a list is returned in that case.
 #' @param gen.iii Logical scalar. If \code{TRUE}, invoke \code{\link{gen_iii}} 
-#'   on each plate.
+#'   on each plate. This is automatically done with CSV input if the plate 
+#'   type is given as \sQuote{OTH} (which is usually the case for plates run 
+#'   in ID mode).
 #'
 #' @param include Pattern for selecting from the input files. The default value
 #'   results in the output of \code{\link{file_pattern}}, which should be
@@ -451,7 +481,7 @@ opm_files <- function(what = c("scripts", "testdata")) {
 #' @family IO-functions
 #' @references \url{http://www.yaml.org/}
 #' @references \url{http://www.biolog.com/}
-#' @seealso read.csv yaml::yaml.load_file
+#' @seealso utils::read.csv yaml::yaml.load_file
 #' @keywords IO
 #' @examples
 #' test.files <- grep("Multiple", opm_files("testdata"), invert = TRUE, 
@@ -507,6 +537,7 @@ read_opm <- function(names, convert = c("try", "no", "yes"), gen.iii = FALSE,
 }
 
 
+################################################################################
 ################################################################################
 #
 # Metadata IO
@@ -515,28 +546,36 @@ read_opm <- function(names, convert = c("try", "no", "yes"), gen.iii = FALSE,
 
 setGeneric("to_metadata",
   function(object, ...) standardGeneric("to_metadata"))
-#' Input metadata (filename version)
+#' Input metadata
 #'
-#' Read metadata from an input file. This is only a thin wrapper of
-#' \code{read.delim} but contains some useful adaptations (such as \strong{not}
-#' converting strings to factors, and not modifying column names).
+#' The character method reads metadata from an input file and is only a thin
+#' wrapper for \code{read.delim} but contains some useful adaptations (such as
+#' \strong{not} converting strings to factors, and not modifying column names).
+#' The default method
+#' reads metadata from an object convertible to a dataframe and is only a thin
+#' wrapper of \code{as.data.frame} but contains the same useful adaptations as
+#' the filename method.
 #'
-#' @name to_metadata,character
-#'
-#' @param object Name of input file (character scalar).
+#' @param object Name of input file (character scalar), or any object 
+#'   convertible to a dataframe.
+#' @param stringsAsFactors Logical scalar passed to \code{as.data.frame}.
+#' @param optional Logical scalar passed to \code{as.data.frame} or
+#'   \code{read.delim}.
 #' @param sep Character scalar. Field separator in input file. This and the
 #'   following parameters are passed to \code{read.delim}.
 #' @param check.names Logical scalar.
 #' @param strip.white Logical scalar.
-#' @param stringsAsFactors Logical scalar.
-#' @param ... Optional other arguments for \code{read.delim}.
+#' @param ... Optional other arguments for \code{read.delim} or 
+#'   \code{as.data.frame}.
 #' @export
 #' @return Dataframe.
 #' @family metadata-functions
 #' @family IO-functions
-#' @keywords IO
-#' @seealso default.stringsAsFactors read.delim
+#' @keywords IO manip
+#' @seealso base::default.stringsAsFactors utils::read.delim base::as.data.frame
 #' @examples
+#'
+#' # Character method
 #' (x <- to_metadata(list(a = 7:8, `b c` = letters[1:2])))
 #' tmpfile <- tempfile()
 #' write.table(x, tmpfile, row.names = FALSE, sep = "\t")
@@ -544,6 +583,12 @@ setGeneric("to_metadata",
 #' (x2 <- to_metadata(tmpfile))
 #' stopifnot(identical(names(x2), names(x)), !identical(names(x1), names(x)))
 #'
+#' # Default method
+#' x <- list(a = 7:8, `b c` = letters[1:2])
+#' (x1 <- as.data.frame(x))
+#' (x2 <- to_metadata(x))
+#' stopifnot(!identical(names(x), names(x1)), identical(names(x), names(x2)))
+#'
 setMethod("to_metadata", "character", function(object,
     sep = "\t", check.names = FALSE, strip.white = TRUE,
     stringsAsFactors = FALSE, ...) {
@@ -552,29 +597,7 @@ setMethod("to_metadata", "character", function(object,
     strip.white = strip.white, stringsAsFactors = stringsAsFactors, ...)
 }, sealed = SEALED)
 
-
-#' Input metadata
-#'
-#' Read metadata from an object convertible to a dataframe. This is only a thin
-#' wrapper of \code{as.data.frame} but contains some useful adaptations, such 
-#' as \strong{not} converting strings to factors, and not modifying column 
-#' names by default.
-#'
-#' @param object Object convertible to a dataframe.
-#' @param stringsAsFactors Logical scalar passed to \code{as.data.frame}.
-#' @param optional Logical scalar passed to \code{as.data.frame}.
-#' @param ... Optional arguments passed to \code{as.data.frame}.
-#' @return Dataframe.
 #' @export
-#' @family metadata-functions
-#' @family IO-functions
-#' @keywords manip
-#' @seealso default.stringsAsFactors as.data.frame
-#' @examples
-#' x <- list(a = 7:8, `b c` = letters[1:2])
-#' (x1 <- as.data.frame(x))
-#' (x2 <- to_metadata(x))
-#' stopifnot(!identical(names(x), names(x1)), identical(names(x), names(x2)))
 #'
 setMethod("to_metadata", "ANY", function(object, stringsAsFactors = FALSE, 
     optional = TRUE, ...) {
@@ -583,6 +606,7 @@ setMethod("to_metadata", "ANY", function(object, stringsAsFactors = FALSE,
 }, sealed = SEALED)
 
 
+################################################################################
 ################################################################################
 #
 # Batch-collection functions
@@ -634,16 +658,26 @@ batch_collect <- function(names, fun, fun.args = list(), ...,
 }
 
 
+################################################################################
+
+
+setGeneric("collect_template",
+  function(object, ...) standardGeneric("collect_template"))
 #' Collect metadata template
 #'
-#' Batch-collect information from OmniLog(R) CSV comments for later on adding
-#' metadata. Optionally add these as novel rows to previously collected data.
-#' Write collected template to a file for use with an external editor, and/or
-#' create data frame for editing the data directly in R with the \code{edit} 
-#' function.
+#' Collect a metadata template from OmniLog(R) CSV comments assisting in later
+#' on adding metadata using  \code{\link{include_metadata}}.
+#' The character method batch-collects such information from files and
+#' optionally add these data as novel rows to previously collected data.
+#' It writes the collected template to a file for use with an external editor,
+#' and/or creates a data frame for editing the data directly in R with the
+#' \code{edit} function.
+#' The \code{\link{OPM}} and \code{\link{OPMS}} methods collect a dataframe.
 #'
-#' @param object Character vector. Acts like the \code{names} argument of 
-#'   \code{\link{read_opm}}. That is, if it a directory name, this is 
+#' @param object Character vector or \code{\link{OPM}} or \code{\link{OPMS}} 
+#'   object. If a character vector is provided, it acts like the \code{names} 
+#'   argument of 
+#'   \code{\link{read_opm}}. That is, if it is a directory name, this is 
 #'   automatically scanned for all CSV and YAML files it contains (unless
 #'   restrictions with patterns are made). One can also provide file names, or
 #'   a mixture of file and directory names.
@@ -655,33 +689,41 @@ batch_collect <- function(names, fun, fun.args = list(), ...,
 #'   two distinct names for novel and old files; see \code{previous}.
 #' @param sep Character scalar. CSV field separator for \code{outfile}.
 #' @param previous Ignored if \code{NULL}. Otherwise passed to 
-#'   \code{\link{to_metadata}} or \code{\link{to_metadata,character}}. If it is
-#'   a filename different from \code{outfile}, it is an error if the file does
-#'   not exist.
+#'   \code{\link{to_metadata}}. If it is a filename different from
+#'   \code{outfile}, it is an error if the file does not exist.
 #' @param md.args List of other arguments passed to the \sQuote{to_metadata}
 #'   methods.
 #' @param add.cols Optional character vector with the names of columns to be 
-#'   added to the result. See the eponymous argument of 
-#'   \code{\link{collect_template,OPM}} for details.
+#'   added to the result, or \code{NULL}. If not empty, names of 
+#'   columns to be added, initially containing \code{NA}.
 #' @param selection Elements to be extracted from the CSV comments contained
-#'   in each file. See \code{\link{collect_template,OPM}} for details.
+#'   in each file. Character vector passed to \code{\link{csv_data}}.
 #' @param include File inclusion pattern (or generator for a pattern). Passed
 #'   to \code{\link{batch_collect}}.
-#' @param ... Other arguments passed to \code{\link{batch_collect}}.
+#' @param ... Other arguments passed to \code{\link{batch_collect}}, or
+#'   arguments passed from the \code{\link{OPMS}} to the \code{\link{OPM}}
+#'   method.
 #' @param demo Logical scalar. Run in \sQuote{demo} mode? Also passed to
 #'   \code{\link{batch_collect}}.
 #' @export
-#' @return Dataframe, returned invisibly if \code{outfile} is given; if 
+#' @return In the case of the character method, a
+#'   dataframe, returned invisibly if \code{outfile} is given; if 
 #'   \code{demo} is \code{TRUE}, a character vector of filenames instead, also
-#'   returned invisibly.
+#'   returned invisibly. The \code{\link{OPM}} method returns a dataframe with
+#'   one row and the number of columns equal to the sum of the lengths of 
+#'   \code{selection} and \code{add.cols}. The \code{\link{OPM}} method returns 
+#'   such a dataframe with one row per contained plate.
+#'
 #' @note Regarding the CSV format, see the remark to 
 #'   \code{\link{read_single_opm}}.
-#' @seealso edit read.delim
+#' @seealso utils::edit utils::read.delim
 #' @family IO-functions
 #' @family metadata-functions
 #' @references \url{http://www.biolog.com/}
-#' @keywords IO
+#' @keywords IO attribute
 #' @examples 
+#'
+#' # Character method
 #' test.files <- grep("Multiple", opm_files("testdata"), invert = TRUE, 
 #'   value = TRUE, fixed = TRUE)
 #' if (length(test.files) > 0) {
@@ -704,6 +746,23 @@ batch_collect <- function(names, fun, fun.args = list(), ...,
 #'   unlink(outfile)
 #' }
 #'
+#' # OPM method
+#' data(vaas_1)
+#' (x <- collect_template(vaas_1))
+#' stopifnot(identical(dim(x), c(1L, 3L)))
+#' (x <- collect_template(vaas_1, add.cols = c("A", "B")))
+#' stopifnot(identical(dim(x), c(1L, 5L)))
+#' # see include_metadata() for how to use this to add metadata information
+#'
+#' # OPMS method
+#' data(vaas_4)
+#' (x <- collect_template(vaas_4))
+#' stopifnot(identical(dim(x), c(4L, 3L)))
+#' (x <- collect_template(vaas_4, add.cols = c("A", "B")))
+#' stopifnot(identical(dim(x), c(4L, 5L)))
+#' # again see include_metadata() for how to use this to add metadata 
+#' # information
+#'
 setMethod("collect_template", "character", function(object, outfile = NULL,
     sep = "\t", previous = outfile, md.args = list(),
     selection = c(SETUP, POS, FILE), add.cols = NULL, 
@@ -741,7 +800,29 @@ setMethod("collect_template", "character", function(object, outfile = NULL,
   }
 }, sealed = SEALED)
 
+#' @export
+#'
+setMethod("collect_template", OPM, function(object, 
+    selection = c(SETUP, POS, FILE), add.cols = NULL) {
+  result <- as.list(csv_data(object, selection))
+  result <- as.data.frame(result, stringsAsFactors = FALSE, optional = TRUE)
+  if (length(add.cols) > 0L) {
+    to.add <- matrix(nrow = nrow(result), ncol = length(add.cols), 
+      dimnames = list(NULL, add.cols), data = NA_character_)
+    result <- cbind(result, to.add, stringsAsFactors = FALSE)
+  }
+  result
+}, sealed = SEALED)
 
+#' @export
+#'
+setMethod("collect_template", OPMS, function(object, ...) {
+  result <- lapply(object@plates, collect_template, ...)
+  do.call(rbind, result)
+}, sealed = SEALED)
+
+
+################################################################################
 ################################################################################
 #
 # Batch conversion functions
@@ -838,6 +919,9 @@ process_io <- function(files, io.fun, fun.args = list(),
 }
 
 
+################################################################################
+
+
 ## NOTE: not an S4 method because conversion is done
 
 #' Convert infiles to outfiles
@@ -850,7 +934,7 @@ process_io <- function(files, io.fun, fun.args = list(),
 #' @inheritParams batch_collect
 #' @param out.ext Character scalar. The extension of the outfile names (without
 #'   the dot).
-#' @param proc Integer scalar. The number pf processes to spawn.
+#' @param proc Integer scalar. The number of processes to spawn.
 #' @param outdir Character vector. Directories in which to place the outfiles.
 #'   If \code{NULL}, each infile's directory is used.
 #' @param in.ext Character scalar. Passed through \code{\link{file_pattern}},
@@ -913,6 +997,7 @@ batch_process <- function(names, out.ext, io.fun, fun.args = list(), proc = 1L,
 }
 
 
+################################################################################
 ################################################################################
 #
 # Batch IO with \code{\link{OPM}} objects
@@ -961,7 +1046,7 @@ batch_process <- function(names, out.ext, io.fun, fun.args = list(), proc = 1L,
 #' @family IO-functions
 #' @references \url{http://www.yaml.org/}
 #' @references \url{http://www.biolog.com/}
-#' @seealso read.csv yaml::yaml.load_file
+#' @seealso utils::read.csv yaml::yaml.load_file
 #' @keywords IO
 #'
 #' @details
@@ -1063,6 +1148,7 @@ batch_opm_to_yaml <- function(names, md.args = NULL, aggr.args = NULL,
 }
 
 
+################################################################################
 ################################################################################
 #
 # File splitting
@@ -1102,6 +1188,8 @@ batch_opm_to_yaml <- function(names, md.args = NULL, aggr.args = NULL,
 #'   that first needs to be converted to a regular expression?
 #' @param invert Logical scalar. Invert pattern matching, i.e. treat all lines
 #'   that \strong{not} match \code{pattern} as separators?
+#' @param include Logical scalar. Also include the separator lines in the 
+#'   output files?
 #'
 #' @param format Character scalar determining the outfile name format. It is
 #'   passed to \code{sprintf} and expects three placeholders: (i) the basename
@@ -1122,7 +1210,7 @@ batch_opm_to_yaml <- function(names, md.args = NULL, aggr.args = NULL,
 #'   list is returned invisibly.
 #'
 #' @family IO-functions
-#' @seealso split strsplit
+#' @seealso base::split base::strsplit
 #' @keywords utilities
 #' @examples
 #' # Splitting an old-style CSV file containing several plates
@@ -1146,8 +1234,8 @@ batch_opm_to_yaml <- function(names, md.args = NULL, aggr.args = NULL,
 #' ## This is used by the run_opm.R script
 #'
 split_files <- function(files, pattern, outdir = "", demo = FALSE, 
-    single = TRUE, wildcard = FALSE, invert = FALSE, format = "%s-%05i.%s", 
-    compressed = TRUE, ...) {
+    single = TRUE, wildcard = FALSE, invert = FALSE, include = TRUE,
+    format = "%s-%05i.%s", compressed = TRUE, ...) {
   
   create_outnames <- function(files, compressed, outdir) {
     file.pat <- file_pattern("any", compressed = compressed)
@@ -1160,8 +1248,8 @@ split_files <- function(files, pattern, outdir = "", demo = FALSE,
     list(base = out.base, ext = out.ext)
   }
 
-  assert_length(pattern, outdir, demo, single, wildcard, invert, format, 
-    compressed)
+  assert_length(pattern, outdir, demo, single, wildcard, invert, include,
+    format, compressed)
   files <- unique(as.character(files))
   out <- create_outnames(files, compressed = compressed, outdir = outdir)
   if (wildcard)
@@ -1170,7 +1258,7 @@ split_files <- function(files, pattern, outdir = "", demo = FALSE,
   invisible(mapply(function(infile, out.base, out.ext) {
     data <- readLines(con = infile)
     data <- split(data, group_by_sep(object = data, pattern = pattern, 
-      invert = invert, ...))
+      invert = invert, include = include, ...))
     if ((len <- length(data)) == 0L || (!single && len == 1L))
       return(character())
     outnames <- sprintf(format, out.base, seq_along(data), out.ext)
@@ -1184,6 +1272,7 @@ split_files <- function(files, pattern, outdir = "", demo = FALSE,
 }
 
 
+################################################################################
 ################################################################################
 #
 # File renaming
@@ -1204,9 +1293,9 @@ split_files <- function(files, pattern, outdir = "", demo = FALSE,
 #'   be modified.
 #' @param overwrite Logical scalar. Overwrite pre-existing files, and do not
 #'   care for duplicate names created by cleaning the filenames?
-#' @param demo Logical scalar. Do not rename files, just return the usual
-#'   result indicating the renamings that would be attempted? (Note that this
-#'   does not indicate whether the renamings would also by successful.)
+#' @param demo Logical scalar. Do not rename files but just return the usual
+#'   result indicating the renaming actions that would be attempted? (Note that
+#'   this does not indicate whether the renaming would also by successful.)
 #' @param empty.tmpl Character scalar. The template to use for filenames that
 #'   become empty after cleaning. Should include an integer placeholder to
 #'   enable incrementing an index for creating unique filenames. (Empty 
@@ -1216,7 +1305,7 @@ split_files <- function(files, pattern, outdir = "", demo = FALSE,
 #'   values corresponding to the novel names, returned invisibly.
 #' @family IO-functions
 #' @keywords utilities
-#' @seealso file.rename
+#' @seealso base::file.rename
 #' @examples
 #'  
 #' # Check the example files: they should be ok
@@ -1265,3 +1354,7 @@ clean_filenames <- function(x, overwrite = FALSE, demo = FALSE,
 }
 
                    
+################################################################################
+
+
+                   
